package com.cxytiandi.api.cache;

import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;
import com.cxytiandi.api.common.ResponseData;
import com.cxytiandi.api.util.JsonUtils;

@Aspect
@Component
public class CacheAspect {
	private static Logger logger = LoggerFactory.getLogger(CacheAspect.class);
	
	@Autowired
	private StringRedisTemplate stringRedisTemplate;
	
	@Around("execution(* com.cxytiandi.api.controller.*.*(..))")
	public Object around(ProceedingJoinPoint joinPoint) throws Throwable {
		long startTime = System.currentTimeMillis();
		Class<?> cls = joinPoint.getTarget().getClass();
		String methodName = joinPoint.getSignature().getName();
		Map<String, Object> map = isChache(cls, methodName);
		boolean isCache = (boolean)map.get("isCache");
		if (isCache) {
			long timeOut = (long) map.get("timeOut");
			TimeUnit timeUnit = (TimeUnit) map.get("timeUnit");
			StringBuilder sb = new StringBuilder();
			Object[] args = joinPoint.getArgs();
			for (Object object : args) {
				sb.append(object);
			}
			String key = cls.getSimpleName() + "_" + methodName + "_" + sb.toString();
			try {
				if (stringRedisTemplate.hasKey(key)) {
					String value = stringRedisTemplate.opsForValue().get(key);
					ResponseData responseData = JsonUtils.toBean(ResponseData.class, value);
					long endTime = System.currentTimeMillis();
					long time = endTime - startTime;
					logger.info(cls.getName() + "." + methodName + "耗时:" + time  + " 缓存数据");
					return responseData;
				} else {
					Object result = joinPoint.proceed();
					if (timeOut == 0) {
						stringRedisTemplate.opsForValue().set(key, JsonUtils.toJson(result));
					} else {
						stringRedisTemplate.opsForValue().set(key, JsonUtils.toJson(result), timeOut, timeUnit);
					}
					long endTime = System.currentTimeMillis();
					long time = endTime - startTime;
					logger.info(cls.getName() + "." + methodName + "耗时:" + time  + " 新增缓存数据");
					return result;
				}
			} catch (Exception e) {
				//报警处理
			}
		}
		Object re = joinPoint.proceed();
		long endTime = System.currentTimeMillis();
		long time = endTime - startTime;
		logger.info(cls.getName() + "." + methodName + "耗时:" + time );
		return re;
	}
	
	private Map<String, Object> isChache(Class<?> cls, String methodName) {
		boolean isCache = false;
		Map<String, Object> map = new HashMap<>();
		Method[] methods = cls.getDeclaredMethods();
		for (Method method : methods) {
			if (method.getName().equals(methodName) && method.isAnnotationPresent(Cache.class)) {
				Cache cache = method.getAnnotation(Cache.class);
				map.put("timeOut", cache.timeOut());
				map.put("timeUnit", cache.timeUnit());
				isCache = true;
				break;
			}
		}
		map.put("isCache", isCache);
		return map;
	}
}
