package com.cxytiandi.api.controller;

import java.util.Date;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.cxytiandi.api.bean.Auth;
import com.cxytiandi.api.common.ResponseData;
import com.cxytiandi.api.document.ApiUser;
import com.cxytiandi.api.exception.ParamException;
import com.cxytiandi.api.service.AuthService;
import com.cxytiandi.api.util.RSAUtils;

@RestController
@RequestMapping("/rest/auth")
public class AuthController {
	
	@Autowired
	private AuthService authService;
	
	@GetMapping
	public ResponseData auth(String accessKey, String secreKey) throws Exception {
		if (!StringUtils.hasText(accessKey) || !StringUtils.hasText(secreKey)) {
			throw new ParamException("accessKey and secreKey is not null");
		}
		
		ApiUser apiUser = authService.getAPiUser(accessKey, secreKey);
		if (apiUser == null) {
			return ResponseData.fail("认证失败");
		}
		
		Auth auth = new Auth();
		long endTime = new Date().getTime() + 1000 * 10;
		String token = apiUser.getUid() + "_" + endTime;
		auth.setToken(RSAUtils.encryptByPublicKey(token));
		return ResponseData.ok(auth);
	}
}
