package com.cxytiandi.api.controller;

import java.util.concurrent.ExecutionException;
import java.util.concurrent.Semaphore;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicLong;

import org.springframework.aop.aspectj.AspectJAdviceParameterNameDiscoverer.AmbiguousBindingException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.cxytiandi.api.cache.Cache;
import com.cxytiandi.api.cache.NoAuth;
import com.cxytiandi.api.common.ResponseData;
import com.cxytiandi.api.exception.GlabalException;
import com.cxytiandi.api.exception.LimitException;
import com.cxytiandi.api.exception.ServerException;
import com.cxytiandi.api.service.HelloService;
import com.google.common.cache.CacheBuilder;
import com.google.common.cache.CacheLoader;
import com.google.common.cache.LoadingCache;
import com.google.common.util.concurrent.RateLimiter;

@RestController
@RequestMapping(value="/rest/hello", produces = "application/json; charset=UTF-8")
public class HelloController {
	
	@Autowired
	private HelloService helloService;
	
	@GetMapping("/{name}")
	public ResponseData hello(@PathVariable String name) throws GlabalException {
		/*if (true) {
			throw new ParamException("参数不能为空");
		}
		*/
		try {
			//return ResponseData.ok(new Hello("yinjihuan"), "success");
			return ResponseData.ok(helloService.hello(name));
		} catch (Exception e) {
			//return ResponseData.fail(e.getMessage());
			throw new ServerException(e.getMessage());
		}
	}
	
	private Semaphore eemaphore = new Semaphore(1);
	
	@GetMapping("/h/{name}")
	@Cache(timeOut=60, timeUnit=TimeUnit.SECONDS)
	public ResponseData hello2(@PathVariable String name) throws InterruptedException {
		eemaphore.acquire();
		System.out.println("测试缓存");
		Thread.sleep(50);
		eemaphore.release();
		return ResponseData.ok(name);
	}
	
	private RateLimiter rateLimiter = RateLimiter.create(1);
	
	private LoadingCache<Long, AtomicLong> counter = CacheBuilder.newBuilder()
			.expireAfterWrite(2, TimeUnit.SECONDS)
			.build(new CacheLoader<Long, AtomicLong>(){
				public AtomicLong load(Long arg0) throws Exception {
					return new AtomicLong(0);
				};
			});
	
	@GetMapping("/hh/{name}")
	@Cache(timeOut=60, timeUnit=TimeUnit.SECONDS)
	public ResponseData hello3(@PathVariable String name) throws Exception {
		//rateLimiter.acquire();
		long sceonds = System.currentTimeMillis() / 1000;
		if (counter.get(sceonds).incrementAndGet() > 10) {
			throw new LimitException("速度太快");
		}
		//Thread.sleep(50);
		return ResponseData.ok(name);
	}
	
	@NoAuth
	@GetMapping("/user")
	public ResponseData user() {
		return ResponseData.ok("yjh");
	}
}
