package com.cxytiandi.api.filter;

import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.springframework.context.ApplicationContext;
import org.springframework.util.StringUtils;
import org.springframework.web.context.support.WebApplicationContextUtils;
import com.cxytiandi.api.cache.ApiWhiteData;
import com.cxytiandi.api.common.ResponseData;
import com.cxytiandi.api.document.ApiUser;
import com.cxytiandi.api.service.ApiUserService;
import com.cxytiandi.api.util.IpUtils;
import com.cxytiandi.api.util.JsonUtils;
import com.cxytiandi.api.util.RSAUtils;

public class HttpBasicAuthorizeFilter implements Filter {
	
	private ApiUserService apiUserService;
	
	@Override
	public void init(FilterConfig filterConfig) throws ServletException {
		ServletContext context = filterConfig.getServletContext();
		ApplicationContext ctx = WebApplicationContextUtils.getWebApplicationContext(context);
		apiUserService = (ApiUserService) ctx.getBean("ApiUserService");
	}

	@Override
	public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain)
			throws IOException, ServletException {
		HttpServletRequest req = (HttpServletRequest) request;
		HttpServletResponse resp = (HttpServletResponse) response;
		System.out.println(IpUtils.getIpAddr(req));
		resp.setContentType("application/json;charset=utf-8");
		System.out.println("-----------" + req.getRequestURI());
		String auth = req.getHeader("Authorization");
		if (ApiWhiteData.getWhiteApis().contains(req.getRequestURI())) {
			chain.doFilter(req, resp);
			return ;
		} else {
			if (!StringUtils.hasText(auth)) {
				PrintWriter print = resp.getWriter();
				print.write(JsonUtils.toJson(ResponseData.fail("非法请求，缺少Authorization头信息")));
				return ;
			} else {
				try {
					String token = RSAUtils.decryptByPrivateKey(auth);
					System.out.println(token);
					String[] ts = token.split("_");
					String uid = ts[0];
					long endTime = Long.parseLong(ts[1]);
					if (System.currentTimeMillis() > endTime) {
						PrintWriter print = resp.getWriter();
						print.write(JsonUtils.toJson(ResponseData.fail("请求失败，token已过期，请重新申请", 300)));
						return ;
					}
					
					ApiUser apiUser = apiUserService.getAPiUser(uid);
					if (apiUser == null) {
						PrintWriter print = resp.getWriter();
						print.write(JsonUtils.toJson(ResponseData.fail("请求失败，用户不存在")));
						return ;
					}
					
					Long limitCount = apiUserService.getUseCount(uid, apiUser.getLimitCount());
					if (limitCount < 1) {
						PrintWriter print = resp.getWriter();
						print.write(JsonUtils.toJson(ResponseData.fail("请求失败，今日API请求次数已用完")));
						return ;
					}
					apiUserService.setUseCount(uid);
				} catch (Exception e) {
					PrintWriter print = resp.getWriter();
					print.write(JsonUtils.toJson(ResponseData.fail("非法请求"+e.getMessage())));
					return ;
				}
			}
		}
		chain.doFilter(req, resp);
	}

	@Override
	public void destroy() {
		
	}

}
