package com.cxytiandi.api.listener;

import java.io.IOException;
import java.lang.reflect.Method;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.context.event.ApplicationPreparedEvent;
import org.springframework.context.ApplicationListener;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;

import com.cxytiandi.api.cache.ApiWhiteData;
import com.cxytiandi.api.cache.NoAuth;
import com.cxytiandi.api.util.ClasspathPackageScannerUtils;

/**
 * spring boot上下文context创建完成
 * @author yinjihuan
 *
 */
public class ApplicationListenerPrepared implements ApplicationListener<ApplicationPreparedEvent> {
	private Logger logger = LoggerFactory.getLogger(ApplicationListenerPrepared.class);
	
	@Override
	public void onApplicationEvent(ApplicationPreparedEvent arg0) {
		try {
			initNoAuthApi();
		} catch (ClassNotFoundException | IOException e) {
			logger.error("", e);
		}
	}
	
	private void initNoAuthApi() throws IOException, ClassNotFoundException {
		ClasspathPackageScannerUtils scan = new ClasspathPackageScannerUtils("com.cxytiandi.api.controller");
		List<String> classList = scan.getFullyQualifiedClassNameList();
		for (String clazz : classList) {
			Class<?> clz = Class.forName(clazz);
			String basePath = "";
			if (clz.isAnnotationPresent(RequestMapping.class)) {
				basePath = clz.getAnnotation(RequestMapping.class).value()[0];
			}
			
			Method[] methods = clz.getDeclaredMethods();
			for (Method method : methods) {
				if (method.isAnnotationPresent(NoAuth.class) && method.isAnnotationPresent(RequestMapping.class)) {
					String[] values = method.getAnnotation(RequestMapping.class).value();
					String value = values.length > 0 ? values[0] : "";
					String uri = basePath + value;
					logger.info("不需要验证的接口：" + uri);
					ApiWhiteData.add(uri);
				}
				if (method.isAnnotationPresent(NoAuth.class) && method.isAnnotationPresent(PostMapping.class)) {
					String[] values = method.getAnnotation(PostMapping.class).value();
					String value = values.length > 0 ? values[0] : "";
					String uri = basePath + value;
					logger.info("不需要验证的接口：" + uri);
					ApiWhiteData.add(uri);
				}
				if (method.isAnnotationPresent(NoAuth.class) && method.isAnnotationPresent(GetMapping.class)) {
					String[] values = method.getAnnotation(GetMapping.class).value();
					String value = values.length > 0 ? values[0] : "";
					String uri = basePath + value;
					logger.info("不需要验证的接口：" + uri);
					ApiWhiteData.add(uri);
				}
			}
		}
	}
}