package com.cxytiandi.api.service;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.concurrent.TimeUnit;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import com.cxytiandi.api.document.ApiUser;
import com.cxytiandi.api.repository.ApiUserRepository;

@Service("ApiUserService")
public class ApiUserServiceImpl implements ApiUserService {
	
	@Autowired
	private ApiUserRepository apiUserRepository;
	
	@Autowired
	private StringRedisTemplate stringRedisTemplate;
	
	@Override
	public ApiUser getAPiUser(String uid) {
		return apiUserRepository.findTop1ByUid(uid);
	}

	@Override
	public long getUseCount(String uid, long limitCount) {
		String date = new SimpleDateFormat("yyyyMMdd").format(new Date());
		String key = uid + date;
		try {
			String value = stringRedisTemplate.opsForValue().get(key);
			if (value == null) {
				stringRedisTemplate.opsForValue().set(key, limitCount+"", 1, TimeUnit.DAYS);
				return limitCount;
			}
			return Long.parseLong(value);
		} catch (Exception e) {
			return 100;
		}
	}

	@Override
	public void setUseCount(String uid) {
		String date = new SimpleDateFormat("yyyyMMdd").format(new Date());
		String key = uid + date;
		try {
			stringRedisTemplate.opsForValue().increment(key, -1);
		} catch (Exception e) {
			// TODO: handle exception
		}
	}
	
}
