package com.cxytiandi.api.util;

import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.http.client.SimpleClientHttpRequestFactory;
import org.springframework.http.converter.FormHttpMessageConverter;
import org.springframework.http.converter.HttpMessageConverter;
import org.springframework.http.converter.StringHttpMessageConverter;
import org.springframework.http.converter.json.MappingJackson2HttpMessageConverter;
import org.springframework.web.client.DefaultResponseErrorHandler;
import org.springframework.web.client.RestTemplate;

import com.cxytiandi.api.common.ResponseData;

public class RestTemplateUtil {
	private static RestTemplate restTemplate;
	private static String token = null;
	static {
        SimpleClientHttpRequestFactory requestFactory = new SimpleClientHttpRequestFactory();
        requestFactory.setReadTimeout(10000);
        requestFactory.setConnectTimeout(10000);

        // 添加转换器
        List<HttpMessageConverter<?>> messageConverters = new ArrayList<HttpMessageConverter<?>>();
        messageConverters.add(new StringHttpMessageConverter(Charset.forName("UTF-8")));
        messageConverters.add(new FormHttpMessageConverter());
        messageConverters.add(new MappingJackson2HttpMessageConverter());

        restTemplate = new RestTemplate(messageConverters);
        restTemplate.setRequestFactory(requestFactory);
        restTemplate.setErrorHandler(new DefaultResponseErrorHandler());
    }
	
	public ResponseData get(String url, Map<String, Object> params) {
		return request(url, params, HttpMethod.GET, false);
	}
	
	public ResponseData post(String url, Map<String, Object> params) {
		return request(url, params, HttpMethod.POST, false);
	}
	
	public ResponseData put(String url, Map<String, Object> params) {
		return request(url, params, HttpMethod.PUT, false);
	}
	
	public ResponseData delete(String url, Map<String, Object> params) {
		return request(url, params, HttpMethod.DELETE, false);
	}
	
	private ResponseData request(String url, Map<String, Object> params, HttpMethod method, Boolean isReload) {
		HttpHeaders httpHeaders = new HttpHeaders();
		httpHeaders.add("Authorization", getToken(isReload));
		HttpEntity<String> entity = new HttpEntity<>(null, httpHeaders);
		ResponseEntity<ResponseData> rss = null;
		if (params != null) {
			rss = restTemplate.exchange(url, method, entity, ResponseData.class, params);
		} else {
			rss = restTemplate.exchange(url, method, entity, ResponseData.class);
		}
		
		//token过期
		if (rss.getBody().getCode() == 300) {
			return request(url, params, method, true);
		}
		return rss.getBody();
	}
	
	private String getToken(Boolean isReload){
		if (token == null || isReload) {
			System.out.println("重新申请token");
			ResponseData responseData = restTemplate.getForObject("http://localhost:8888/rest/auth?accessKey=111&secreKey=222",
					ResponseData.class);
			if (responseData.getStatus()) {
				Map<String, String> map = (Map<String, String>) responseData.getData();
				String r = map.get("token");
				token = r;
				return r;
			}
		}
		
		return token;
	}
	
}
